<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4202\Controller;

use Eccube\Controller\AbstractController;
use Plugin\ForIbn4202\Entity\Ibn4202Eyecatch;
use Plugin\ForIbn4202\Form\Type\Ibn4202EyecatchType;
use Plugin\ForIbn4202\Repository\Ibn4202EyecatchRepository;
use Eccube\Event\EccubeEvents;
use Eccube\Event\EventArgs;
use Eccube\Util\CacheUtil;
use Knp\Component\Pager\Paginator;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

//▼画像アップロード
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpKernel\Exception\UnsupportedMediaTypeHttpException;
use Symfony\Component\HttpFoundation\Response;

class Ibn4202EyecatchController extends AbstractController
{
    /**
     * @var Ibn4202EyecatchRepository
     */
    protected $ibn4202EyecatchRepository;

    /**
     * Ibn4202EyecatchController constructor.
     *
     * @param Ibn4202EyecatchRepository $ibn4202EyecatchRepository
     */
    public function __construct(Ibn4202EyecatchRepository $ibn4202EyecatchRepository)
    {
        $this->ibn4202EyecatchRepository = $ibn4202EyecatchRepository;
    }

    /**
     * 一覧を表示する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch", name="admin_content_ibn4202_eyecatch")
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/page/{page_no}", requirements={"page_no" = "\d+"}, name="admin_content_ibn4202_eyecatch_page")
     * @Template("@ForIbn4202/admin/ibn4202_eyecatch.twig")
     *
     * @param Request $request
     * @param int $page_no
     * @param Paginator $paginator
     *
     * @return array
     */
    public function index(Request $request)
    {
        $Ibn4202Eyecatchs = $this->ibn4202EyecatchRepository
            ->findBy(
                [],
                ['sort_no' => 'DESC']
            );

        $event = new EventArgs(
            [
                'Ibn4202Eyecatchs' => $Ibn4202Eyecatchs,
            ],
            $request
        );

        return [
            'Ibn4202Eyecatchs' => $Ibn4202Eyecatchs,
        ];
    }

    /**
     * 登録・編集する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/new", name="admin_content_ibn4202_eyecatch_new")
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/{id}/edit", requirements={"id" = "\d+"}, name="admin_content_ibn4202_eyecatch_edit")
     * @Template("@ForIbn4202/admin/ibn4202_eyecatch_edit.twig")
     *
     * @param Request $request
     * @param null $id
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function edit(Request $request, Ibn4202Eyecatch $Ibn4202Eyecatch = null, CacheUtil $cacheUtil)
    {
        if (is_null($Ibn4202Eyecatch)) {
            $Ibn4202Eyecatch = $this->ibn4202EyecatchRepository->findOneBy([], ['sort_no' => 'DESC']);
            $sortNo = 1;
            if ($Ibn4202Eyecatch) {
                $sortNo = $Ibn4202Eyecatch->getSortNo() + 1;
            }

            $Ibn4202Eyecatch = new \Plugin\ForIbn4202\Entity\Ibn4202Eyecatch();
            $Ibn4202Eyecatch
                ->setSortNo($sortNo)
                ->setFixed(true)
                ->setVisible(true);
        }

        $builder = $this->formFactory
            ->createBuilder(Ibn4202EyecatchType::class, $Ibn4202Eyecatch);
        $event = new EventArgs(
            [
                'builder' => $builder,
                'Ibn4202Eyecatch' => $Ibn4202Eyecatch,
            ],
            $request
        );

        $form = $builder->getForm();
		
        //▼画像アップロード
		// 既に画像保存されてる場合は取得する
        $oldIbn4202EyecatchImage01 = $Ibn4202Eyecatch->getImage01();
        $oldIbn4202EyecatchImage02 = $Ibn4202Eyecatch->getImage02();
        $oldIbn4202EyecatchImage03 = $Ibn4202Eyecatch->getImage03();
        $oldIbn4202EyecatchImage04 = $Ibn4202Eyecatch->getImage04();
        $oldIbn4202EyecatchImage05 = $Ibn4202Eyecatch->getImage05();

        $form->setData($Ibn4202Eyecatch);
		//▲

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

			//▼画像アップロード
            $Ibn4202Eyecatch = $form->getData();
            // ファイルアップロード
            $fs = new Filesystem();

            $file01 = $form['image01']->getData();
            if ($file01 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file01)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file01,
                    $this->getParameter('eccube_save_image_dir').'/'.$file01
                );
            }
            $file02 = $form['image02']->getData();
            if ($file02 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file02)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file02,
                    $this->getParameter('eccube_save_image_dir').'/'.$file02
                );
            }
            $file03 = $form['image03']->getData();
            if ($file03 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file03)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file03,
                    $this->getParameter('eccube_save_image_dir').'/'.$file03
                );
            }
            $file04 = $form['image04']->getData();
            if ($file04 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file04)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file04,
                    $this->getParameter('eccube_save_image_dir').'/'.$file04
                );
            }
            $file05 = $form['image05']->getData();
            if ($file05 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file05)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file05,
                    $this->getParameter('eccube_save_image_dir').'/'.$file05
                );
            }
			//▲

            $this->ibn4202EyecatchRepository->save($Ibn4202Eyecatch);

            $event = new EventArgs(
                [
                    'form' => $form,
                    'Ibn4202Eyecatch' => $Ibn4202Eyecatch,
                ],
                $request
            );

            $this->addSuccess('admin.common.save_complete', 'admin');

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

            return $this->redirectToRoute('admin_content_ibn4202_eyecatch_edit', ['id' => $Ibn4202Eyecatch->getId()]);
        }

        return [
            'form' => $form->createView(),
            'Ibn4202Eyecatch' => $Ibn4202Eyecatch,
			//▼画像アップロード
            'oldIbn4202EyecatchImage01' => $oldIbn4202EyecatchImage01,
            'oldIbn4202EyecatchImage02' => $oldIbn4202EyecatchImage02, 
            'oldIbn4202EyecatchImage03' => $oldIbn4202EyecatchImage03, 
            'oldIbn4202EyecatchImage04' => $oldIbn4202EyecatchImage04, 
            'oldIbn4202EyecatchImage05' => $oldIbn4202EyecatchImage05, 
        ];
    }

    //▼画像アップロード
	/**
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/image/process", name="admin_ibn4202_eyecatch_image_process")
     */
    public function imageProcess(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $images = $request->files->get('admin_ibn4202_eyecatch'); //Ibn4202EyecatchType.phpの最下部、return 'admin_ibn4202_eyecatch'
        $allowExtensions = ['gif', 'jpg', 'jpeg', 'png'];
        $filename = null;
		
        if (isset($images['image01_file'])) {
            $image = $images['image01_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        if (isset($images['image02_file'])) {
            $image = $images['image02_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        if (isset($images['image03_file'])) {
            $image = $images['image03_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        if (isset($images['image04_file'])) {
            $image = $images['image04_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        if (isset($images['image05_file'])) {
            $image = $images['image05_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        return new Response($filename);
    }

	// ▼画像アップロード
    /**
     * アップロード画像を取得する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#load
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/image/load", name="admin_ibn4202_eyecatch_image_load")
     */
    public function imageLoad(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $dirs = [
            $this->eccubeConfig['eccube_save_image_dir'],
            $this->eccubeConfig['eccube_temp_image_dir'],
        ];

        foreach ($dirs as $dir) {
            $image = \realpath($dir.'/'.$request->query->get('source'));
            $dir = \realpath($dir);

            if (\is_file($image) && \str_starts_with($image, $dir)) {
                $file = new \SplFileObject($image);

                return $this->file($file, $file->getBasename());
            }
        }

        throw new NotFoundHttpException();
    }

	// ▼画像アップロード
    /**
     * アップロード画像をすぐ削除する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#revert
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/image/revert", name="admin_ibn4202_eyecatch_image_revert")
     */
    public function imageRevert(Request $request)
    {
        if (!$request->isXmlHttpRequest() && $this->isTokenValid()) {
            throw new BadRequestHttpException();
        }

        $tempFile = $this->eccubeConfig['eccube_temp_image_dir'].'/'.$request->getContent();
        if (is_file($tempFile) && stripos(realpath($tempFile), $this->eccubeConfig['eccube_temp_image_dir']) === 0) {
            $fs = new Filesystem();
            $fs->remove($tempFile);

            return new Response(null, Response::HTTP_NO_CONTENT);
        }

        throw new NotFoundHttpException();
    }

    /**
     * 指定したデータを削除する。
     *
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/{id}/delete", requirements={"id" = "\d+"}, name="admin_content_ibn4202_eyecatch_delete", methods={"DELETE"})
     *
     * @param Request $request
     * @param Ibn4202Eyecatch $Ibn4202Eyecatch
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function delete(Request $request, Ibn4202Eyecatch $Ibn4202Eyecatch, CacheUtil $cacheUtil)
    {
        $this->isTokenValid();

        log_info('新入荷商品特集削除開始', [$Ibn4202Eyecatch->getId()]);

        try {
            $this->ibn4202EyecatchRepository->delete($Ibn4202Eyecatch);

            $this->addSuccess('admin.common.delete_complete', 'admin');

            log_info('新入荷商品特集削除完了', [$Ibn4202Eyecatch->getId()]);

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

        } catch (\Exception $e) {
            $message = trans('admin.common.delete_error_foreign_key', ['%name%' => $Ibn4202Eyecatch->getTopImage()]);
            $this->addError($message, 'admin');

            log_error('新入荷商品特集削除エラー', [$Ibn4202Eyecatch->getId(), $e]);
        }

        $sortNo = 1;
        $Ibn4202Eyecatchs = $this->ibn4202EyecatchRepository
            ->findBy([], ['sort_no' => 'ASC']);

        foreach ($Ibn4202Eyecatchs as $Ibn4202Eyecatch) {
                $Ibn4202Eyecatch->setSortNo($sortNo);
                $sortNo++;
        }
        $this->entityManager->flush();

        return $this->redirectToRoute('admin_content_ibn4202_eyecatch');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/{id}/visibility", requirements={"id" = "\d+"}, name="admin_content_ibn4202_eyecatch_visibility", methods={"PUT"})
     */
    public function visibility(Request $request, Ibn4202Eyecatch $Ibn4202Eyecatch)
    {
        $this->isTokenValid();

        // 表示・非表示を切り替える
        if ($Ibn4202Eyecatch->isVisible()) {
            $message = trans('admin.common.to_hide_complete', ['%name%' => $Ibn4202Eyecatch->getId()]);
            $Ibn4202Eyecatch->setVisible(false);
        } else {
            $message = trans('admin.common.to_show_complete', ['%name%' => $Ibn4202Eyecatch->getId()]);
            $Ibn4202Eyecatch->setVisible(true);
        }
        $this->entityManager->persist($Ibn4202Eyecatch);

        $this->entityManager->flush();

        $this->addSuccess($message, 'admin');

        return $this->redirectToRoute('admin_content_ibn4202_eyecatch');
    }

    /**
     * @Route("/%eccube_admin_route%/content/ibn4202_eyecatch/sort_no/move", name="admin_content_ibn4202_eyecatch_sort_no_move", methods={"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function moveSortNo(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        if ($this->isTokenValid()) {
            $sortNos = $request->request->all();
            foreach ($sortNos as $ibn4202EyecatchId => $sortNo) {
                $Ibn4202Eyecatch = $this->ibn4202EyecatchRepository->find($ibn4202EyecatchId);
                $Ibn4202Eyecatch->setSortNo($sortNo);
                $this->entityManager->persist($Ibn4202Eyecatch);
            }
            $this->entityManager->flush();
        }

        return $this->json('OK', 200);
    }

}
