<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4202\Controller;

use Eccube\Controller\AbstractController;
use Plugin\ForIbn4202\Entity\ToppageSlide;
use Eccube\Event\EccubeEvents;
use Eccube\Event\EventArgs;
use Plugin\ForIbn4202\Form\Type\ToppageSlideType;
use Plugin\ForIbn4202\Repository\ToppageSlideRepository;
use Eccube\Util\CacheUtil;
use Knp\Component\Pager\PaginatorInterface;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

//▼画像アップロード
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpKernel\Exception\UnsupportedMediaTypeHttpException;
use Symfony\Component\HttpFoundation\Response;

class ToppageSlideController extends AbstractController
{
    /**
     * @var ToppageSlideRepository
     */
    protected $toppageSlideRepository;

    /**
     * ToppageSlideController constructor.
     *
     * @param ToppageSlideRepository $toppageSlideRepository
     */
    public function __construct(ToppageSlideRepository $toppageSlideRepository)
    {
        $this->toppageSlideRepository = $toppageSlideRepository;
    }

    /**
     * 一覧を表示する。
     *
     * @Route("/%eccube_admin_route%/plugin/toppage_slide", name="plugin_toppage_slide")
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/page/{page_no}", requirements={"page_no" = "\d+"}, name="plugin_toppage_slide_page")
     * @Template("@ForIbn4202/admin/toppage_slide.twig")
     *
     * @param Request $request
     * @param int $page_no
     * @param Paginator $paginator
     *
     * @return array
     */
    public function index(Request $request, PaginatorInterface $paginator, $page_no = 1)
    {
        $qb = $this->toppageSlideRepository
            ->findBy(
                [],
                ['sort_no' => 'DESC']
            );

        $pagination = $paginator->paginate(
            $qb,
            $page_no,
            $this->eccubeConfig->get('eccube_default_page_count')
        );

        return [
            'pagination' => $pagination,
        ];
    }

    /**
     * 登録・編集する。
     *
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/new", name="plugin_toppage_slide_new")
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/{id}/edit", requirements={"id" = "\d+"}, name="plugin_toppage_slide_edit")
     * @Template("@ForIbn4202/admin/toppage_slide_edit.twig")
     *
     * @param Request $request
     * @param null $id
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function edit(Request $request, ToppageSlide $ToppageSlide = null, CacheUtil $cacheUtil)
    {
        if (is_null($ToppageSlide)) {
            $ToppageSlide = $this->toppageSlideRepository->findOneBy([], ['sort_no' => 'DESC']);
            $sortNo = 1;
            if ($ToppageSlide) {
                $sortNo = $ToppageSlide->getSortNo() + 1;
            }

            $ToppageSlide = new \Plugin\ForIbn4202\Entity\ToppageSlide();
            $ToppageSlide
                ->setSortNo($sortNo)
                ->setFixed(true)
                ->setVisible(true);
        }

        $builder = $this->formFactory
            ->createBuilder(ToppageSlideType::class, $ToppageSlide);

        $form = $builder->getForm();
		
        //▼画像アップロード
		// 既に画像保存されてる場合は取得する
        $oldImage01 = $ToppageSlide->getImage01();
        $oldImage02 = $ToppageSlide->getImage02();
        $form->setData($ToppageSlide);
		//▲

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

			//▼画像アップロード
            $ToppageSlide = $form->getData();
            // ファイルアップロード
            $fs = new Filesystem();
			
            $file01 = $form['image01']->getData();
            if ($file01 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file01)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file01,
                    $this->getParameter('eccube_save_image_dir').'/'.$file01
                );
            }
            $file02 = $form['image02']->getData();
            if ($file02 && $fs->exists($this->getParameter('eccube_temp_image_dir').'/'.$file02)) {
                $fs->rename(
                    $this->getParameter('eccube_temp_image_dir').'/'.$file02,
                    $this->getParameter('eccube_save_image_dir').'/'.$file02
                );
            }

            if (!$ToppageSlide->getUrl()) {
                $ToppageSlide->setLinkMethod(false);
            }
			//▲
			
            $this->toppageSlideRepository->save($ToppageSlide);

            $this->addSuccess('admin.common.save_complete', 'admin');

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

            return $this->redirectToRoute('plugin_toppage_slide_edit', ['id' => $ToppageSlide->getId()]);
        }

        return [
            'form' => $form->createView(),
            'ToppageSlide' => $ToppageSlide,
            'oldImage01' => $oldImage01, //▼画像アップロード
            'oldImage02' => $oldImage02, 
        ];
    }

    //▼画像アップロード
	/**
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/image/process", name="admin_toppage_slide_image_process")
     */
    public function imageProcess(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $images = $request->files->get('admin_toppage_slide'); //ToppageSlideType.phpの最下部、return 'admin_toppage_slide'
        $allowExtensions = ['gif', 'jpg', 'jpeg', 'png'];
        $filename = null;
		
        if (isset($images['image01_file'])) {
            $image = $images['image01_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }
		
        if (isset($images['image02_file'])) {
            $image = $images['image02_file'];

            //ファイルフォーマット検証
            $mimeType = $image->getMimeType();
            if (0 !== strpos($mimeType, 'image')) {
                throw new UnsupportedMediaTypeHttpException();
            }

            // 拡張子
            $extension = $image->getClientOriginalExtension();
            if (!in_array($extension, $allowExtensions)) {
                throw new UnsupportedMediaTypeHttpException();
            }

            $filename = date('mdHis').uniqid('_').'.'.$extension;
            $image->move($this->getParameter('eccube_temp_image_dir'), $filename);
        }

        return new Response($filename);
    }

	// ▼画像アップロード
    /**
     * アップロード画像を取得する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#load
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/image/load", name="admin_toppage_slide_image_load")
     */
    public function imageLoad(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        $dirs = [
            $this->eccubeConfig['eccube_save_image_dir'],
            $this->eccubeConfig['eccube_temp_image_dir'],
        ];

        foreach ($dirs as $dir) {
            $image = \realpath($dir.'/'.$request->query->get('source'));
            $dir = \realpath($dir);

            if (\is_file($image) && \str_starts_with($image, $dir)) {
                $file = new \SplFileObject($image);

                return $this->file($file, $file->getBasename());
            }
        }

        throw new NotFoundHttpException();
    }

	// ▼画像アップロード
    /**
     * アップロード画像をすぐ削除する際にコールされるメソッド.
     *
     * @see https://pqina.nl/filepond/docs/api/server/#revert
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/image/revert", name="admin_toppage_slide_image_revert")
     */
    public function imageRevert(Request $request)
    {
        if (!$request->isXmlHttpRequest() && $this->isTokenValid()) {
            throw new BadRequestHttpException();
        }

        $tempFile = $this->eccubeConfig['eccube_temp_image_dir'].'/'.$request->getContent();
        if (is_file($tempFile) && stripos(realpath($tempFile), $this->eccubeConfig['eccube_temp_image_dir']) === 0) {
            $fs = new Filesystem();
            $fs->remove($tempFile);

            return new Response(null, Response::HTTP_NO_CONTENT);
        }

        throw new NotFoundHttpException();
    }

    /**
     * 指定したデータを削除する。
     *
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/{id}/delete", requirements={"id" = "\d+"}, name="plugin_toppage_slide_delete", methods={"DELETE"})
     *
     * @param Request $request
     * @param ToppageSlide $ToppageSlide
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function delete(Request $request, ToppageSlide $ToppageSlide, CacheUtil $cacheUtil)
    {
        $this->isTokenValid();

        log_info('トップ画像削除開始', [$ToppageSlide->getId()]);

        try {
            $this->toppageSlideRepository->delete($ToppageSlide);

            $this->addSuccess('admin.common.delete_complete', 'admin');

            log_info('トップ画像削除完了', [$ToppageSlide->getId()]);

            // キャッシュの削除
            $cacheUtil->clearDoctrineCache();

        } catch (\Exception $e) {
            $message = trans('admin.common.delete_error_foreign_key', ['%name%' => $ToppageSlide->getTopImage()]);
            $this->addError($message, 'admin');

            log_error('トップ画像削除エラー', [$ToppageSlide->getId(), $e]);
        }

        $sortNo = 1;
        $ToppageSlides = $this->toppageSlideRepository
            ->findBy([], ['sort_no' => 'ASC']);

        foreach ($ToppageSlides as $ToppageSlide) {
                $ToppageSlide->setSortNo($sortNo);
                $sortNo++;
        }
        $this->entityManager->flush();

        return $this->redirectToRoute('plugin_toppage_slide');
    }

    /**
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/{id}/visibility", requirements={"id" = "\d+"}, name="plugin_toppage_slide_visibility", methods={"PUT"})
     */
    public function visibility(Request $request, ToppageSlide $ToppageSlide)
    {
        $this->isTokenValid();

        // 表示・非表示を切り替える
        if ($ToppageSlide->isVisible()) {
            $message = trans('admin.common.to_hide_complete', ['%name%' => $ToppageSlide->getId()]);
            $ToppageSlide->setVisible(false);
        } else {
            $message = trans('admin.common.to_show_complete', ['%name%' => $ToppageSlide->getId()]);
            $ToppageSlide->setVisible(true);
        }
        $this->entityManager->persist($ToppageSlide);

        $this->entityManager->flush();

        $this->addSuccess($message, 'admin');

        return $this->redirectToRoute('plugin_toppage_slide');
    }

    /**
     * @Route("/%eccube_admin_route%/plugin/toppage_slide/sort_no/move", name="plugin_toppage_slide_sort_no_move", methods={"POST"})
     *
     * @param Request $request
     *
     * @return Response
     */
    public function moveSortNo(Request $request)
    {
        if (!$request->isXmlHttpRequest()) {
            throw new BadRequestHttpException();
        }

        if ($this->isTokenValid()) {
            $sortNos = $request->request->all();
            foreach ($sortNos as $toppageSlideId => $sortNo) {
                $ToppageSlide = $this->toppageSlideRepository->find($toppageSlideId);
                $ToppageSlide->setSortNo($sortNo);
                $this->entityManager->persist($ToppageSlide);
            }
            $this->entityManager->flush();
        }

        return $this->json('OK', 200);
    }

}
