<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4202\Entity;

use Doctrine\ORM\Mapping as ORM;
use Eccube\Entity\AbstractEntity;

if (!class_exists('\Plugin\ForIbn4202\Entity\Ibn4202Category')) {
    /**
     * Ibn4202Category
     *
     * @ORM\Table(name="plg_ibn4202_category")
     * @ORM\InheritanceType("SINGLE_TABLE")
     * @ORM\DiscriminatorColumn(name="discriminator_type", type="string", length=255)
     * @ORM\HasLifecycleCallbacks()
     * @ORM\Entity(repositoryClass="Plugin\ForIbn4202\Repository\Ibn4202CategoryRepository")
     */
    class Ibn4202Category extends AbstractEntity
    {
        /**
         * @var int
         *
         * @ORM\Column(name="id", type="integer", options={"unsigned":true})
         * @ORM\Id
         * @ORM\GeneratedValue(strategy="IDENTITY")
         */
        private $id;

        /**
         * @var string|null
         *
         * @ORM\Column(name="category_image", type="string", length=255, nullable=true)
         */
        private $category_image;

        /**
         * @var string|null
         *
         * @ORM\Column(name="category_text1", type="string", length=255, nullable=true)
         */
        private $category_text1;

		/**
		 * @var int
		 *
		 * @ORM\Column(name="category_id", type="integer", nullable=false, options={"unsigned":true})
		 */
		private $category_id;

        /**
         * @var int|null
         *
         * @ORM\Column(name="sort_no", type="smallint", nullable=true, options={"unsigned":true})
         */
        private $sort_no;

        /**
         * @var boolean
         *
         * @ORM\Column(name="fixed", type="boolean", options={"default":true})
         */
        private $fixed = true;

        /**
         * @var \DateTime
         *
         * @ORM\Column(name="create_date", type="datetimetz")
         */
        private $create_date;

        /**
         * @var \DateTime
         *
         * @ORM\Column(name="update_date", type="datetimetz")
         */
        private $update_date;

        /**
         * @var boolean
         *
         * @ORM\Column(name="visible", type="boolean", options={"default":true})
         */
        private $visible;

        /**
         * @var \Eccube\Entity\Member
         *
         * @ORM\ManyToOne(targetEntity="Eccube\Entity\Member")
         * @ORM\JoinColumns({
         *   @ORM\JoinColumn(name="creator_id", referencedColumnName="id")
         * })
         */
        private $Creator;

        /**
         * Get id.
         *
         * @return int
         */
        public function getId()
        {
            return $this->id;
        }

        /**
         * Set categoryImage.
         *
         * @param string|null $categoryImage
         *
         * @return TopImage
         */
        public function setCategoryImage($categoryImage = null)
        {
            $this->category_image = $categoryImage;

            return $this;
        }

        /**
         * Get categoryImage.
         *
         * @return string|null
         */
        public function getCategoryImage()
        {
            return $this->category_image;
        }

        /**
         * Set categoryText1.
         *
         * @param string|null $categoryText1
         *
         * @return categoryText1
         */
        public function setCategoryText1($categoryText1 = null)
        {
            $this->category_text1 = $categoryText1;

            return $this;
        }

        /**
         * Get categoryText1.
         *
         * @return string|null
         */
        public function getCategoryText1()
        {
            return $this->category_text1;
        }

        /**
         * Set categoryId.
         *
         * @param string|null $categoryId
         *
         * @return categoryId
         */
        public function setCategoryId($categoryId)
        {
            $this->category_id = $categoryId;

            return $this;
        }

        /**
         * Get categoryId.
         *
         * @return string|null
         */
        public function getCategoryId()
        {
            return $this->category_id;
        }

        /**
         * Set sortNo.
         *
         * @param int|null $sortNo
         *
         * @return Payment
         */
        public function setSortNo($sortNo = null)
        {
            $this->sort_no = $sortNo;

            return $this;
        }

        /**
         * Get sortNo.
         *
         * @return int|null
         */
        public function getSortNo()
        {
            return $this->sort_no;
        }

        /**
         * Set fixed.
         *
         * @param boolean $fixed
         *
         * @return Payment
         */
        public function setFixed($fixed)
        {
            $this->fixed = $fixed;

            return $this;
        }

        /**
         * Get fixed.
         *
         * @return boolean
         */
        public function isFixed()
        {
            return $this->fixed;
        }

        /**
         * Set createDate.
         *
         * @param \DateTime $createDate
         *
         * @return Ibn4202Category
         */
        public function setCreateDate($createDate)
        {
            $this->create_date = $createDate;

            return $this;
        }

        /**
         * Get createDate.
         *
         * @return \DateTime
         */
        public function getCreateDate()
        {
            return $this->create_date;
        }

        /**
         * Set updateDate.
         *
         * @param \DateTime $updateDate
         *
         * @return Ibn4202Category
         */
        public function setUpdateDate($updateDate)
        {
            $this->update_date = $updateDate;

            return $this;
        }

        /**
         * Get updateDate.
         *
         * @return \DateTime
         */
        public function getUpdateDate()
        {
            return $this->update_date;
        }

        /**
         * @return integer
         */
        public function isVisible()
        {
            return $this->visible;
        }

        /**
         * @param boolean $visible
         *
         * @return Ibn4202Category
         */
        public function setVisible($visible)
        {
            $this->visible = $visible;

            return $this;
        }

        /**
         * Set creator.
         *
         * @param \Eccube\Entity\Member|null $creator
         *
         * @return Ibn4202Category
         */
        public function setCreator(\Eccube\Entity\Member $creator = null)
        {
            $this->Creator = $creator;

            return $this;
        }

        /**
         * Get creator.
         *
         * @return \Eccube\Entity\Member|null
         */
        public function getCreator()
        {
            return $this->Creator;
        }
    }
}
