<?php

/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) LOCKON CO.,LTD. All Rights Reserved.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\ForIbn4202\Form\Type\Admin;

use Eccube\Common\EccubeConfig;
use Plugin\ForIbn4202\Entity\ForIbn4202Config;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;

/**
 * Class ForIbn4202ConfigType.
 */
class ForIbn4202ConfigType extends AbstractType
{
    /**
     * @var EccubeConfig
     */
    protected $eccubeConfig;

    /**
     * ForIbn4202ConfigType constructor.
     *
     * @param EccubeConfig $eccubeConfig
     */
    public function __construct(EccubeConfig $eccubeConfig)
    {
        $this->eccubeConfig = $eccubeConfig;
    }

    /**
     * Build form.
     *
     * @param FormBuilderInterface $builder
     * @param array                $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('eyecatch_title', TextType::class, [
                'required' => false,
            ])
            ->add('topic_title', TextType::class, [
                'required' => false,
            ])
            ->add('new_item_title', TextType::class, [
                'required' => false,
            ])
            ->add('category_title', TextType::class, [
                'required' => false,
            ])
            ->add('newitem_sort', ChoiceType::class, [
                'constraints' => [
                    new Assert\NotBlank(),
                ],
                'choices' => array(
                    '登録順' => '0',
                    '更新順' => '1'
                ),
                'multiple'  => false,
                'expanded' => true,
            ])
            ->add('newitem_count', IntegerType::class, [
                'constraints' => [
                    new Assert\NotBlank(),
                ],
            ])
        ;
    }

    /**
     * Config.
     *
     * @param OptionsResolver $resolver
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults([
            'data_class' => ForIbn4202Config::class,
        ]);
    }
}
